@@ -1,61 +1,23 @@
-from mocker import MockerTestCase
+# This file is part of cloud-init. See LICENSE file for license information.
 
 from cloudinit import cloud
+from cloudinit.config import cc_growpart
 from cloudinit import util
 
-from cloudinit.config import cc_growpart
+from cloudinit.tests.helpers import TestCase
 
 import errno
 import logging
 import os
 import re
 import unittest
+from contextlib import ExitStack
+from unittest import mock
 
 # growpart:
-#   mode: auto  # off, on, auto, 'growpart', 'parted'
+#   mode: auto  # off, on, auto, 'growpart'
 #   devices: ['root']
 
-HELP_PARTED_NO_RESIZE = """
-Usage: parted [OPTION]... [DEVICE [COMMAND [PARAMETERS]...]...]
-Apply COMMANDs with PARAMETERS to DEVICE.  If no COMMAND(s) are given, run in
-interactive mode.
-
-OPTIONs:
-<SNIP>
-
-COMMANDs:
-<SNIP>
-  quit                                     exit program
-  rescue START END                         rescue a lost partition near START
-        and END
-  resize NUMBER START END                  resize partition NUMBER and its file
-        system
-  rm NUMBER                                delete partition NUMBER
-<SNIP>
-Report bugs to bug-parted@gnu.org
-"""
-
-HELP_PARTED_RESIZE = """
-Usage: parted [OPTION]... [DEVICE [COMMAND [PARAMETERS]...]...]
-Apply COMMANDs with PARAMETERS to DEVICE.  If no COMMAND(s) are given, run in
-interactive mode.
-
-OPTIONs:
-<SNIP>
-
-COMMANDs:
-<SNIP>
-  quit                                     exit program
-  rescue START END                         rescue a lost partition near START
-        and END
-  resize NUMBER START END                  resize partition NUMBER and its file
-        system
-  resizepart NUMBER END                    resize partition NUMBER
-  rm NUMBER                                delete partition NUMBER
-<SNIP>
-Report bugs to bug-parted@gnu.org
-"""
-
 HELP_GROWPART_RESIZE = """
 growpart disk partition
    rewrite partition table so that partition takes up all the space it can
@@ -83,8 +45,20 @@ growpart disk partition
       Resize partition 1 on /dev/sda
 """
 
+HELP_GPART = """
+usage: gpart add -t type [-a alignment] [-b start] <SNIP> geom
+       gpart backup geom
+       gpart bootcode [-b bootcode] [-p partcode -i index] [-f flags] geom
+<SNIP>
+       gpart resize -i index [-a alignment] [-s size] [-f flags] geom
+       gpart restore [-lF] [-f flags] provider [...]
+       gpart recover [-f flags] geom
+       gpart help
+<SNIP>
+"""
 
-class TestDisabled(MockerTestCase):
+
+class TestDisabled(unittest.TestCase):
     def setUp(self):
         super(TestDisabled, self).setUp()
         self.name = "growpart"
@@ -95,18 +69,18 @@ class TestDisabled(MockerTestCase):
         self.handle = cc_growpart.handle
 
     def test_mode_off(self):
-        #Test that nothing is done if mode is off.
+        # Test that nothing is done if mode is off.
 
         # this really only verifies that resizer_factory isn't called
         config = {'growpart': {'mode': 'off'}}
-        self.mocker.replace(cc_growpart.resizer_factory,
-                            passthrough=False)
-        self.mocker.replay()
 
-        self.handle(self.name, config, self.cloud_init, self.log, self.args)
+        with mock.patch.object(cc_growpart, 'resizer_factory') as mockobj:
+            self.handle(self.name, config, self.cloud_init, self.log,
+                        self.args)
+            self.assertEqual(mockobj.call_count, 0)
 
 
-class TestConfig(MockerTestCase):
+class TestConfig(TestCase):
     def setUp(self):
         super(TestConfig, self).setUp()
         self.name = "growpart"
@@ -114,84 +88,96 @@ class TestConfig(MockerTestCase):
         self.cloud = cloud.Cloud(None, self.paths, None, None, None)
         self.log = logging.getLogger("TestConfig")
         self.args = []
-        os.environ = {}
 
         self.cloud_init = None
         self.handle = cc_growpart.handle
 
-        # Order must be correct
-        self.mocker.order()
-
-    @unittest.skip("until LP: #1212444 fixed")
+    @mock.patch.dict("os.environ", clear=True)
     def test_no_resizers_auto_is_fine(self):
-        subp = self.mocker.replace(util.subp, passthrough=False)
-        subp(['parted', '--help'], env={'LANG': 'C'})
-        self.mocker.result((HELP_PARTED_NO_RESIZE, ""))
-        subp(['growpart', '--help'], env={'LANG': 'C'})
-        self.mocker.result((HELP_GROWPART_NO_RESIZE, ""))
-        self.mocker.replay()
+        with mock.patch.object(
+                util, 'subp',
+                return_value=(HELP_GROWPART_NO_RESIZE, "")) as mockobj:
 
-        config = {'growpart': {'mode': 'auto'}}
-        self.handle(self.name, config, self.cloud_init, self.log, self.args)
+            config = {'growpart': {'mode': 'auto'}}
+            self.handle(self.name, config, self.cloud_init, self.log,
+                        self.args)
 
+            mockobj.assert_has_calls([
+                mock.call(['growpart', '--help'], env={'LANG': 'C'}),
+                mock.call(['gpart', 'help'], env={'LANG': 'C'}, rcs=[0, 1])])
+
+    @mock.patch.dict("os.environ", clear=True)
     def test_no_resizers_mode_growpart_is_exception(self):
-        subp = self.mocker.replace(util.subp, passthrough=False)
-        subp(['growpart', '--help'], env={'LANG': 'C'})
-        self.mocker.result((HELP_GROWPART_NO_RESIZE, ""))
-        self.mocker.replay()
+        with mock.patch.object(
+                util, 'subp',
+                return_value=(HELP_GROWPART_NO_RESIZE, "")) as mockobj:
+            config = {'growpart': {'mode': "growpart"}}
+            self.assertRaises(
+                ValueError, self.handle, self.name, config,
+                self.cloud_init, self.log, self.args)
 
-        config = {'growpart': {'mode': "growpart"}}
-        self.assertRaises(ValueError, self.handle, self.name, config,
-                          self.cloud_init, self.log, self.args)
+            mockobj.assert_called_once_with(
+                ['growpart', '--help'], env={'LANG': 'C'})
 
-    @unittest.skip("until LP: #1212444 fixed")
-    def test_mode_auto_prefers_parted(self):
-        subp = self.mocker.replace(util.subp, passthrough=False)
-        subp(['parted', '--help'], env={'LANG': 'C'})
-        self.mocker.result((HELP_PARTED_RESIZE, ""))
-        self.mocker.replay()
+    @mock.patch.dict("os.environ", clear=True)
+    def test_mode_auto_prefers_growpart(self):
+        with mock.patch.object(
+                util, 'subp',
+                return_value=(HELP_GROWPART_RESIZE, "")) as mockobj:
+            ret = cc_growpart.resizer_factory(mode="auto")
+            self.assertIsInstance(ret, cc_growpart.ResizeGrowPart)
 
-        ret = cc_growpart.resizer_factory(mode="auto")
-        self.assertTrue(isinstance(ret, cc_growpart.ResizeParted))
+            mockobj.assert_called_once_with(
+                ['growpart', '--help'], env={'LANG': 'C'})
+
+    @mock.patch.dict("os.environ", clear=True)
+    def test_mode_auto_falls_back_to_gpart(self):
+        with mock.patch.object(
+                util, 'subp',
+                return_value=("", HELP_GPART)) as mockobj:
+            ret = cc_growpart.resizer_factory(mode="auto")
+            self.assertIsInstance(ret, cc_growpart.ResizeGpart)
+
+            mockobj.assert_has_calls([
+                mock.call(['growpart', '--help'], env={'LANG': 'C'}),
+                mock.call(['gpart', 'help'], env={'LANG': 'C'}, rcs=[0, 1])])
 
     def test_handle_with_no_growpart_entry(self):
-        #if no 'growpart' entry in config, then mode=auto should be used
+        # if no 'growpart' entry in config, then mode=auto should be used
 
         myresizer = object()
+        retval = (("/", cc_growpart.RESIZE.CHANGED, "my-message",),)
 
-        factory = self.mocker.replace(cc_growpart.resizer_factory,
-                                      passthrough=False)
-        rsdevs = self.mocker.replace(cc_growpart.resize_devices,
-                                     passthrough=False)
-        factory("auto")
-        self.mocker.result(myresizer)
-        rsdevs(myresizer, ["/"])
-        self.mocker.result((("/", cc_growpart.RESIZE.CHANGED, "my-message",),))
-        self.mocker.replay()
+        with ExitStack() as mocks:
+            factory = mocks.enter_context(
+                mock.patch.object(cc_growpart, 'resizer_factory',
+                                  return_value=myresizer))
+            rsdevs = mocks.enter_context(
+                mock.patch.object(cc_growpart, 'resize_devices',
+                                  return_value=retval))
+            mocks.enter_context(
+                mock.patch.object(cc_growpart, 'RESIZERS',
+                                  (('mysizer', object),)
+                                  ))
 
-        try:
-            orig_resizers = cc_growpart.RESIZERS
-            cc_growpart.RESIZERS = (('mysizer', object),)
             self.handle(self.name, {}, self.cloud_init, self.log, self.args)
-        finally:
-            cc_growpart.RESIZERS = orig_resizers
+
+            factory.assert_called_once_with('auto')
+            rsdevs.assert_called_once_with(myresizer, ['/'])
 
 
-class TestResize(MockerTestCase):
+class TestResize(unittest.TestCase):
     def setUp(self):
         super(TestResize, self).setUp()
         self.name = "growpart"
         self.log = logging.getLogger("TestResize")
 
-        # Order must be correct
-        self.mocker.order()
-
     def test_simple_devices(self):
-        #test simple device list
+        # test simple device list
         # this patches out devent2dev, os.stat, and device_part_info
         # so in the end, doesn't test a lot
         devs = ["/dev/XXda1", "/dev/YYda2"]
-        devstat_ret = Bunch(st_mode=25008, st_ino=6078, st_dev=5L,
+        devstat_ret = Bunch(st_mode=25008, st_ino=6078, st_dev=5,
                             st_nlink=1, st_uid=0, st_gid=6, st_size=0,
                             st_atime=0, st_mtime=0, st_ctime=0)
         enoent = ["/dev/NOENT"]
@@ -233,9 +219,9 @@ class TestResize(MockerTestCase):
                              find("/dev/YYda2", resized)[1])
             self.assertEqual(cc_growpart.RESIZE.SKIPPED,
                              find(enoent[0], resized)[1])
-            #self.assertEqual(resize_calls,
-                             #[("/dev/XXda", "1", "/dev/XXda1"),
-                              #("/dev/YYda", "2", "/dev/YYda2")])
+            # self.assertEqual(resize_calls,
+            #                 [("/dev/XXda", "1", "/dev/XXda1"),
+            #                  ("/dev/YYda", "2", "/dev/YYda2")])
         finally:
             cc_growpart.device_part_info = opinfo
             os.stat = real_stat
@@ -248,9 +234,7 @@ def simple_device_part_info(devpath):
     return x
 
 
-class Bunch:
-    st_mode = None  # fix pylint complaint
-
+class Bunch(object):
     def __init__(self, **kwds):
         self.__dict__.update(kwds)
 
